from typing import Tuple
from nonebot import get_plugin_config, require, on_command, get_bot, get_driver, logger
from nonebot.plugin import PluginMetadata

from .config import Config
from requests import get, HTTPError
import time

require("nonebot_plugin_apscheduler")

from nonebot_plugin_apscheduler import scheduler


__plugin_meta__ = PluginMetadata(
    name="nonebot-plugin-lynchpined",
    description="A nonebot2 plugin to check Lynchpin progress.",
    usage="Get Lynchpin progress by sending 'lynchpin' command. Subscribe to daily Lynchpin progress by adding `lynchpined_user` or `lynchpined_group` in config",
    type="application",
    homepage="https://github.com/theTeamFuture/nonebot-plugin-lynchpined",
    supported_adapters={"~onebot.v11"},
    config=Config,
)

config = Config.parse_obj(get_driver().config)

LYNCHPIN_DEST = "https://ak.hypergryph.com/lynchpin/api/meta"

PATTERN = [0,4,12,15,24,33,34,35,35]
EPOCH = 1715529600

def get_progress()->int:
    response = get(LYNCHPIN_DEST)
    if response.status_code == 200:
        result = response.json()
        return result['data']['progress']
    else:
        raise HTTPError

def get_expected_progress(timestamp)->Tuple[int,int]:
    sec_diff = timestamp - EPOCH
    days = sec_diff // 86400
    days_in_period = days % len(PATTERN)
    return PATTERN[days_in_period], days_in_period

def get_msg()->str:
    progress = get_progress()
    t = int(time.time())
    expected_progress, days_in_period = get_expected_progress(t)
    comparison = 'Matched' if progress == expected_progress else 'UNMATCHED'
    msg = f'Lynchpin: {progress}%\nPattern {comparison}:\n'
    result = [f'{p:2}' if idx!=days_in_period else f'[{p:2}]' for idx,p in enumerate(PATTERN) ]
    msg += ' '.join(result)
    return msg

GET_LYNCHPINED = on_command("lynchpin", priority=10, block=True)

@GET_LYNCHPINED.handle()
async def show_progress(event, state):
    try:
        msg = get_msg()
        await GET_LYNCHPINED.send(msg)
    except HTTPError:
        await GET_LYNCHPINED.send('Lynchpin: HTTPError')


async def run_everyday():
    bot = get_bot()
    msg = get_msg()
    for group_id in config.lynchpined_group:
        await bot.send_group_msg(group_id=group_id, message=msg)
    
    for user_id in config.lynchpined_user:
        await bot.send_msg(user_id=user_id, message=msg)

    logger.info('Daily Lynchpin completed.')

    


scheduler.add_job(func=run_everyday,trigger="cron",second=3,minute=0,hour=0,id="lynchpin_subscribe")






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_lynchpined']

package_data = \
{'': ['*']}

install_requires = \
['nonebot-plugin-apscheduler>=0.4.0,<0.5.0', 'requests>=2.32.3,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-lynchpined',
    'version': '0.1.0',
    'description': 'A nonebot2 plugin to check Lynchpin progress.',
    'long_description': '<div align="center">\n  \n# nonebot-plugin-lynchpined\n\n*Getting lynchpined everyday!*\n\n</div>\n\n## Description\n\n`nonebot-plugin-lynchpined` is a [nonebot2](https://nonebot.dev/) plugin to help you track [lynchpin progress](https://ak.hypergryph.com/lynchpin) in QQ group.\n\n## Installation\n\n### By nb-cli (Recommended)\n\nPending.\n\n```bash\nnb plugin install nonebot-plugin-lynchpined\n```\n\n### By pip\n\n```bash\npip install nonebot-plugin-lynchpined\n```\n\nand then adding following content in the `pyproject.toml`:\n\n```toml\n[tool.nonebot]\nplugins = [\n  ... # others\n  "nonebot-plugin-lynchpined",\n  ]\n```\n\n## Simple Usage\n\nJust call `lynchpin` anywhere bot can chat.\n\n```\n> Lynchpined\nLynchpin: 24%\nPattern Matched:\n 0  4 12 15 [24] 33 34 35 35\n```\n\n## Subscribe to Lynchpin Progress\n\nAdding `lynchpined_user` or `lynchpined_group` in config, the plugin will send lynchpin progress to the lynchpined at 00:00 everyday.\n\nFor example: \n\n```Properties\nLYNCHPINED_GROUP = [4121524, 33343535]\nLYNCHPINED_USER = [11708102]\n```\n\n**Note:** Make sure the timezone on your machine is `UTC+8` or you have to manually modified the trigger in `__init__.py`.\n',
    'author': 'Nightsky',
    'author_email': '050644zf@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
